/* Copyright (C) 2005-2009,Unigine Corp. All rights reserved.
 *
 * File:    light_keyline.h
 * Desc:    Light view of keyline system 
 * Version: 0.1
 * Author:  Pavel Kurinnoy <figonet@unigine.com>
 *
 * This file is part of the Unigine engine (http://unigine.com/).
 *
 * Your use and or redistribution of this software in src and / or
 * binary form,with or without modification,is subject to: (i) your
 * ongoing acceptance of and compliance with the terms and conditions of
 * the Unigine License Agreement; and (ii) your inclusion of this notice
 * in any version of this software that you use or redistribute.
 * A copy of the Unigine License Agreement is available by contacting
 * Unigine Corp. at http://unigine.com/
 */

#ifndef __LIGHT_KEYLINE_H__
#define __LIGHT_KEYLINE_H__

#include <scripts/keyline/keyline.h>

namespace LightKeyline {

	/*************************************************************************\
	 *
	 * constants
	 *
	\*************************************************************************/
	
	string UI_FILE = "light_keyline.ui";
	
	/*************************************************************************\
	 *
	 * variables
	 *
	\*************************************************************************/
	
	// gui
	Gui gui;
	UserInterface ui;

	// widgets
	WidgetVBox window;
	//WidgetWindow window;
	WidgetHSlider slider;
	WidgetLabel pos_label;

	// variables
	KeyLine::KeyLineManager keyline_manager;
	
	// callbacks
	string on_set_caption_callback;

	/*************************************************************************\
	 *
	 * public methods
	 *
	\*************************************************************************/
	
	/* initialize keyline window
	 */
	void init(Gui parent_gui) {
		// init window
		gui = parent_gui;
		ui = engine.gui.addUserInterface(gui,UI_FILE);
		
		pos_label.setFontRich(1);
	}
	
	/* shutdown keyline window
	 */
	void shutdown() {
		engine.gui.removeUserInterface(ui);
	}
	
	/* get window visible
	 */
	int isVisible() { return gui.isChild(window); }
	
	/* update window
	 */
	void update() {
		if(!isVisible() || keyline_manager == NULL) return;
		

		// update keyline manager position  
		if(keyline_manager.isPlaying() && (!slider.isFocused() || !gui.getMouseGrab())) { 
			keyline_manager.play();
		}
		update_slider_pos();
	}

	/* callback is called when position caption has to be updated
	 */
	void setOnSetCaptionCallback(string callback) {
		on_set_caption_callback = callback;
	}

	/* show window
	 */
	void show(KeyLine::KeyLineManager manager) {
		if(isVisible()) return;
		if(manager == NULL) throw("LightKeyLine::show(): manager must not be NULL\n");
		
		keyline_manager = manager;
						
		// init slider 
		slider.setMinValue(int(keyline_manager.getMinPos()));
		slider.setMaxValue(int(keyline_manager.getMaxPos()));	
		update_slider_pos();
		
		// show
		gui.addChild(window,GUI_ALIGN_OVERLAP);
	
	}

	/* hide window
	 */
	void hide() {
		if(!isVisible()) return;
		
		// reset callbacks
		on_set_caption_callback = "";
		
		// reset states
		keyline_manager = NULL;
		
		// hide
		gui.removeChild(window);
	}

	/*
	 */
	void update_keyline_ui() {
		
		// update caption
		if(on_set_caption_callback != "") {
			pos_label.setText(call(on_set_caption_callback,keyline_manager.getPos()));
		} else {
			pos_label.setText(string(slider.getValue()));
		}
	}
	
	/*
	 */
	void update_slider_pos() {
		update_keyline_ui();
		if(slider.isFocused() && gui.getMouseGrab()) return;
		slider.removeFocus();
		if(abs(float(keyline_manager.getPos()) - float(slider.getValue())) < 0.5) return;
		slider.setCallbackEnabled(GUI_CHANGED,0);
		slider.setValue(int(keyline_manager.getPos()));
		slider.setCallbackEnabled(GUI_CHANGED,1);
	}

	/*************************************************************************\
	 *
	 * private callback functions
	 *
	\*************************************************************************/

	/* update slider position
	 */
	void slider_pos_changed() {
		keyline_manager.setPos(slider.getValue());
	}
};
#endif
