/* Copyright (C) 2005-2010, Unigine Corp. All rights reserved.
 *
 * File:	tropics_timeline.cpp
 * Desc:	Timeline for tropics
 * Version:	0.5
 * Authors:	Vasiliy Simonov <simonowvv@unigine.com>
 * 			Pavel Kurinnoy <figonet@unigine.com>
 *
 * This file is part of the Unigine engine (http://unigine.com/).
 *
 * Your use and or redistribution of this software in source and / or
 * binary form, with or without modification, is subject to: (i) your
 * ongoing acceptance of and compliance with the terms and conditions of
 * the Unigine License Agreement; and (ii) your inclusion of this notice
 * in any version of this software that you use or redistribute.
 * A copy of the Unigine License Agreement is available by contacting
 * Unigine Corp. at http://unigine.com/
 */

#ifndef __TROPICS_TIMELINE_H__
#define __TROPICS_TIMELINE_H__

#include <scripts/common/dialogs.h>
#include <keyline/params_basic.h>
#include <tropics_params.h>
#include <keyline/keyline_window.h>
#include <light_keyline.h>
namespace TropicsTimeline {
	
	/*************************************************************************\
	 *
	 * const
	 *
	\*************************************************************************/
	
	string FILE_NAME = "demos/tropics/tropics_timeline.keyman";
	#ifndef RELEASE
	int SNAP_STEP = 900; // 15 minutes
	#endif
	
	/*************************************************************************\
	 *
	 * variables
	 *
	\*************************************************************************/
	
	KeyLine::KeyLineManager timeline;
	int play_mode = 1;
	
	WidgetButton environment_button;
	
	/*************************************************************************\
	 *
	 * private functions
	 *
	\*************************************************************************/
	
	/* callback on environment button clicked
	 */
	void environment_button_clicked() {
		// show editor for timeline
		if(environment_button.isToggled()) {
			#ifdef RELEASE
				if(LightKeyline::isVisible()) LightKeyline::hide();
				LightKeyline::setOnSetCaptionCallback("TropicsTimeline::gen_pos_caption");
				LightKeyline::show(timeline);
				if(engine.editor.isLoaded()) LightKeyline::window.setPosition(90.0,75.0);
				else LightKeyline::window.setPosition(90.0,35.0);
			#else
				// close another editor
				if(KeylineWindow::isVisible()) KeylineWindow::hide();
				// open editor for this timeline
				KeylineWindow::setOnSaveCallback("TropicsTimeline::on_save");
				KeylineWindow::setOnResetCallback("TropicsTimeline::on_reset");
				KeylineWindow::setOnHideCallback("TropicsTimeline::on_editor_closed");
				KeylineWindow::setOnSetCaptionCallback("TropicsTimeline::gen_pos_caption");
				KeylineWindow::show(timeline);
				KeylineWindow::setSnapStep(SNAP_STEP);
			#endif
		}
		// close editor for timeline
		else {
			#ifdef RELEASE
				LightKeyline::hide();
			#else
				KeylineWindow::hide();
			#endif
		}
	}
	
	/* save timeline
	 */
	void on_save() {
		timeline.save(FILE_NAME);
	}
	
	/* reload timeline
	 */
	void on_reset() {
		float pos = timeline.getPos();
		timeline.load(FILE_NAME);
		timeline.setPos(pos);
	}
	
	/* callback hits after TimelineWindow closed
	 */
	void on_editor_closed(int is_changed) {
		if(is_changed && DialogMessage::show("Timeline","Save changes?",DialogMessage::YES | DialogMessage::NO) == DialogMessage::YES) {
			on_save();
		} else {
			on_reset();
		}
		environment_button.setToggled(false);
	}
	
	/* generate formatted caption string for KeylineWindow
	 */
	string gen_pos_caption(float pos) {
		int time = int(pos);
		return format("<font color=\"#FF9900\">%d:%02d</font>",(time / 3600) % 24,(time / 60) % 60);
	}
	
	/*************************************************************************\
	 *
	 * public functions
	 *
	\*************************************************************************/
	
	/*
	 */
	int init() {
		KeyLine::registerBasicParams();
		registerTropicsParams();
		
		// load timeline
		timeline = new KeyLine::KeyLineManager();
		timeline.load(FILE_NAME);
	}
	
	/*
	 */
	int shutdown() {
	}
	
	/*
	 */
	int update() {
	
		if((environment_button.isToggled() == 0) && play_mode) {

			timeline.play();
		}
		timeline.apply();
	}
	
	float getTimeSpeed() { return timeline.getMultiplier() * 240.0; }
	void setTimeSpeed(float f) { timeline.setMultiplier(f / 240.0); }
	
	float getTime() { return timeline.getPos() / 3600.0; }
	void setTime(float f) { timeline.setPos(f * 3600.0); }
	
	int getPlayMode() { return play_mode; }
	void setPlayMode(int p) { 
		play_mode = p;
		if (play_mode == 0) timeline.stop();
		else timeline.play();
	}
}

#endif // __TROPICS_TIMELINE_H__
